import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  scaleOrdinal,
  min,
  max,
  axisLeft,
  axisBottom,
  format,
  extent,
} from 'd3';
import ReactDropdown from 'react-dropdown';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { DropDown } from './DropDown.js';
import { ColorLegend } from './ColorLegend.js';

const height = 450;
const width = 955;
const padding = 20;
const centerX = width / 2;
const centerY = height / 2;
const margin = {
  top: 30,
  bottom: 100,
  left: 100,
  right: 100,
};
const tickOffset = 5;
const xAxisLabelOffset = 50;
const yAxisLabelOffset = -150;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const siFormat = () => format('.2s');
const xAxisTickFormat = (tickValue) =>
  siFormat('.2s')(tickValue).replace('G', 'B');

const attributes = [
  { value: 'sepal_length', label: 'Sepal Length' },
  { value: 'sepal_width', label: 'Sepal Width' },
  { value: 'petal_length', label: 'Petal Length' },
  { value: 'petal_width', label: 'Petal Width' },
  { value: 'species', label: 'Species' },
];

const getLabel = (attr) => {
  const label = attributes.filter(
    (pair) => pair.value == attr
  );
  return label[0].label;
};

const App = () => {
  const data = useData();

  // X, Y, and color attribute settings
  const initXAttr = 'petal_length';
  const [xAttr, setXAttr] = useState(initXAttr);
  const xValue = (d) => d[xAttr];
  const xAxisLabel = getLabel(xAttr);
  const initYAttr = 'sepal_width';
  const [yAttr, setYAttr] = useState(initYAttr);
  const yValue = (d) => d[yAttr];
  const yAxisLabel = getLabel(yAttr);
  const colorVal = (d) => d.species;

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  const speciesList = data.map((row) => row.species);
  const speciesTypes = speciesList.filter((element, i) => {
    return speciesList.indexOf(element) != i ? false : true;
  });

  const colorScale = scaleOrdinal()
    .domain(speciesTypes)
    .range(['orange', 'teal', 'purple']);

  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">X </span>
        <ReactDropdown
          className="Dropdown-root"
          options={attributes}
          onChange={({ value }) => setXAttr(value)}
          value={xAttr}
        />
        <span className="dropdown-label">Y </span>
        <ReactDropdown
          className="Dropdown-root"
          options={attributes}
          onChange={({ value }) => setYAttr(value)}
          value={yAttr}
        />
      </div>
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={tickOffset}
          />
          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            tickOffset={tickOffset}
          />
          <text
            x={innerWidth / 2}
            style={{ textAnchor: 'middle' }}
            y={innerHeight + xAxisLabelOffset}
            className="axis-label"
          >
            {xAxisLabel}
          </text>
          <text
            x={yAxisLabelOffset}
            style={{ textAnchor: 'middle' }}
            y={-innerHeight / 2 + 120}
            className="axis-label"
            transform="rotate(-90)"
          >
            {yAxisLabel}
          </text>
          <g transform={`translate(${innerWidth + 20}, 12)`}>
            <text style={{fill: '#635F5D'}}>Species:</text>
            <ColorLegend
              colorScale={colorScale}
              tickSpacing={'24'}
            />
          </g>
          <Marks
            xScale={xScale}
            yScale={yScale}
            colorScale={colorScale}
            data={data}
            xValue={xValue}
            yValue={yValue}
            colorVal={colorVal}
            tooltipFormat={xAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
